@@ -8,11 +8,28 @@ use Icinga\Module\Director\Web\Form\QuickForm;
 
 class ApplyMigrationsForm extends QuickForm
 {
+    /** @var  Migrations */
     protected $migrations;
 
     public function setup()
     {
-        $this->setSubmitLabel($this->translate('Apply schema migrations'));
+        if ($this->migrations->hasSchema()) {
+            $count = $this->migrations->countPendingMigrations();
+            if ($count === 1) {
+                $this->setSubmitLabel(
+                    $this->translate('Apply a pending schema migration')
+                );
+            } else {
+                $this->setSubmitLabel(
+                    sprintf(
+                        $this->translate('Apply %d pending schema migrations'),
+                        $count
+                    )
+                );
+            }
+        } else {
+            $this->setSubmitLabel($this->translate('Create schema'));
+        }
     }
 
     public function onSuccess()
