@@ -9,8 +9,9 @@ use Icinga\Module\Director\IcingaConfig\IcingaConfig;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Objects\IcingaCommand;
 use Icinga\Module\Director\Objects\DirectorDeploymentLog;
+use Icinga\Module\Director\Objects\IcingaZone;
 
-class CoreApi
+class CoreApi implements DeploymentApiInterface
 {
     protected $client;
 
@@ -104,8 +105,8 @@ class CoreApi
     public function getHostOutput($host)
     {
         try {
-            $object = $this->getObject($host);
-        } catch (\Exception $e) {
+            $object = $this->getObject($host, 'hosts');
+        } catch (Exception $e) {
             return 'Unable to fetch the requested object';
         }
         if (isset($object->attrs->last_check_result)) {
@@ -120,7 +121,6 @@ class CoreApi
         $this->checkHostNow($host);
         $now = time();
 
-        $waiting = true;
         while (true) {
             try {
                 $object = $this->getObject($host, 'hosts');
@@ -193,7 +193,7 @@ class CoreApi
             . "\n  if (existing) { return false }"
             . "\n%s\n}\n__run_with_activation_context(f)\n",
             $key,
-            $object->object_name,
+            $object->get('object_name'),
             (string) $object
         );
 
@@ -280,7 +280,7 @@ constants
 
     public function getActiveChecksum(Db $conn)
     {
-        $db = $conn->getConnection();
+        $db = $conn->getDbAdapter();
         $stage = $this->getActiveStageName();
         if (! $stage) {
             return null;
@@ -323,6 +323,9 @@ constants
         return $objects;
     }
 
+    /**
+     * @return IcingaZone[]
+     */
     public function getZoneObjects()
     {
         return $this->getDirectorObjects('Zone', 'Zone', 'zones', array(
@@ -413,7 +416,14 @@ constants
     public function getHostGroupObjects()
     {
         return $this->getDirectorObjects('HostGroup', 'HostGroup', 'hostgroups', array(
-            'display_name'          => 'display_name',
+            'display_name' => 'display_name',
+        ));
+    }
+
+    public function getUserGroupObjects()
+    {
+        return $this->getDirectorObjects('UserGroup', 'UserGroup', 'usergroups', array(
+            'display_name' => 'display_name',
         ));
     }
 
@@ -433,8 +443,24 @@ constants
         }
 
         return $objects;
+    }
 
-        return $this->getObjects('CheckCommand', 'CheckCommands');
+    public function getNotificationCommandObjects()
+    {
+        IcingaCommand::setPluginDir($this->getConstant('PluginDir'));
+
+        $objects = $this->getDirectorObjects('Command', 'NotificationCommand', 'NotificationCommands', array(
+            'arguments' => 'arguments',
+            // 'env'      => 'env',
+            'timeout'   => 'timeout',
+            'command'   => 'command',
+            'vars'      => 'vars'
+        ));
+        foreach ($objects as $obj) {
+            $obj->methods_execute = 'PluginNotification';
+        }
+
+        return $objects;
     }
 
     public function listModuleStages($name, $active = null)
@@ -463,12 +489,11 @@ constants
         return $found;
     }
 
-    public function collectLogFiles($db)
+    public function collectLogFiles(Db $db)
     {
-
         $existing = $this->listModuleStages('director');
         foreach ($db->getUncollectedDeployments() as $deployment) {
-            $stage = $deployment->stage_name;
+            $stage = $deployment->get('stage_name');
             if (! in_array($stage, $existing)) {
                 continue;
             }
@@ -476,9 +501,7 @@ constants
             try {
                 $availableFiles = $this->listStageFiles($stage);
             } catch (Exception $e) {
-                // This is not correct. We might miss logs as af an ongoing reload
-                $deployment->stage_collected = 'y';
-                $deployment->store();
+                // Could not collect stage files. Doesn't matter, let's try next time
                 continue;
             }
 
@@ -486,19 +509,24 @@ constants
                 && in_array('status', $availableFiles)
             ) {
                 if ($this->getStagedFile($stage, 'status') === '0') {
-                    $deployment->startup_succeeded = 'y';
+                    $deployment->set('startup_succeeded', 'y');
                 } else {
-                    $deployment->startup_succeeded = 'n';
+                    $deployment->set('startup_succeeded', 'n');
                 }
-                $deployment->startup_log = $this->getStagedFile($stage, 'startup.log');
+                $deployment->set('startup_log', $this->shortenStartupLog(
+                    $this->getStagedFile($stage, 'startup.log')
+                ));
+            } else {
+                // Stage seems to be incomplete, let's try again next time
+                continue;
             }
-            $collected = true;
+            $deployment->set('stage_collected', 'y');
 
             $deployment->store();
         }
     }
 
-    public function wipeInactiveStages($db)
+    public function wipeInactiveStages(Db $db)
     {
         $uncollected = $db->getUncollectedDeployments();
         $moduleName = 'director';
@@ -586,10 +614,9 @@ constants
         $this->client->request('post', $url, null, false, true);
     }
 
-    public function dumpConfig(IcingaConfig $config, $db, $moduleName = 'director')
+    public function dumpConfig(IcingaConfig $config, Db $db, $moduleName = 'director')
     {
         $start = microtime(true);
-        $data = $config->getFileContents();
         $deployment = DirectorDeploymentLog::create(array(
             // 'config_id'      => $config->id,
             // 'peer_identity'  => $endpoint->object_name,
@@ -613,20 +640,43 @@ constants
 
         $duration = (int) ((microtime(true) - $start) * 1000);
         // $deployment->duration_ms = $duration;
-        $deployment->duration_dump = $duration;
+        $deployment->set('duration_dump', $duration);
 
         if ($response->succeeded()) {
             if ($stage = $response->getResult('stage', array('package' => $moduleName))) { // Status?
-                $deployment->stage_name = key($stage);
-                $deployment->dump_succeeded = 'y';
+                $deployment->set('stage_name', key($stage));
+                $deployment->set('dump_succeeded', 'y');
             } else {
-                $deployment->dump_succeeded = 'n';
+                $deployment->set('dump_succeeded', 'n');
             }
         } else {
-            $deployment->dump_succeeded = 'n';
+            $deployment->set('dump_succeeded', 'n');
         }
 
         $deployment->store($db);
-        return $deployment->dump_succeeded === 'y';
+        return $deployment->set('dump_succeeded', 'y');
+    }
+
+    protected function shortenStartupLog($log)
+    {
+        $logLen = strlen($log);
+        if ($logLen < 1024 * 60) {
+            return $log;
+        }
+
+        $part = substr($log, 0, 1024 * 20);
+        $parts = explode("\n", $part);
+        array_pop($parts);
+        $begin = implode("\n", $parts) . "\n\n";
+
+        $part = substr($log, -1024 * 20);
+        $parts = explode("\n", $part);
+        array_shift($parts);
+        $end = "\n\n" . implode("\n", $parts);
+
+        return $begin . sprintf(
+            '[..] %d bytes removed by Director [..]',
+            $logLen - (strlen($begin) + strlen($end))
+        ) . $end;
     }
 }
