@@ -2,11 +2,10 @@
 
 namespace Icinga\Module\Director;
 
-use Icinga\Data\Db\DbConnection;
+use Icinga\Module\Director\Data\Db\DbConnection;
 use Icinga\Module\Director\Objects\DirectorDeploymentLog;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Module\Director\Objects\IcingaObject;
-use Icinga\Module\Director\Util;
 use Icinga\Exception\ConfigurationError;
 use Zend_Db_Expr;
 use Zend_Db_Select;
@@ -42,7 +41,7 @@ class Db extends DbConnection
 
         if ($object !== null) {
             $query .= $db->quoteInto(' AND object_type = ?', $object->getTableName());
-            $query .= $db->quoteInto(' AND object_name = ?', $object->object_name);
+            $query .= $db->quoteInto(' AND object_name = ?', $object->getObjectName());
         }
         return (int) $db->fetchOne($query);
     }
@@ -61,6 +60,15 @@ class Db extends DbConnection
         return (int) $db->fetchOne($query);
     }
 
+    public function settings()
+    {
+        if ($this->settings === null) {
+            $this->settings = new Settings($this);
+        }
+
+        return $this->settings;
+    }
+
     public function getMasterZoneName()
     {
         if ($this->masterZoneName === null) {
@@ -72,7 +80,7 @@ class Db extends DbConnection
 
     protected function detectMasterZoneName()
     {
-        if ($zone = $this->getSetting('master_zone')) {
+        if ($zone = $this->settings()->master_zone) {
             return $zone;
         }
 
@@ -93,7 +101,7 @@ class Db extends DbConnection
 
     public function getDefaultGlobalZoneName()
     {
-        return $this->getSetting('default_global_zone', 'director-global');
+        return $this->settings()->default_global_zone;
     }
 
     public function hasDeploymentEndpoint()
@@ -147,68 +155,14 @@ class Db extends DbConnection
         return $name;
     }
 
+    /**
+     * @return IcingaEndpoint
+     */
     public function getDeploymentEndpoint()
     {
         return IcingaEndpoint::load($this->getDeploymentEndpointName(), $this);
     }
 
-    public function getSetting($name, $default = null)
-    {
-        if ($this->settings === null) {
-            $this->fetchSettings();
-        }
-
-        if (array_key_exists($name, $this->settings)) {
-            return $this->settings[$name];
-        }
-
-        return $default;
-    }
-
-    public function storeSetting($name, $value)
-    {
-        $db = $this->db();
-        if ($this->getSetting($name) === $value) {
-            return $this;
-        }
-
-        $updated = $db->update(
-            'director_setting',
-            array('setting_value' => $value),
-            $db->quoteInto('setting_name = ?', $name)
-        );
-
-        if ($updated === 0) {
-            $db->insert(
-                'director_setting',
-                array(
-                    'setting_name'  => $name,
-                    'setting_value' => $value,
-                )
-            );
-        }
-
-        if ($this->settings !== null) {
-            $this->settings[$name] = $value;
-        }
-
-        return $this;
-    }
-
-    public function fetchSettings($force = true)
-    {
-        if ($force || $this->settings === null) {
-            $db = $this->db();
-            $query = $db->select()->from(
-                array('s' => 'director_setting'),
-                array('setting_name', 'setting_value')
-            );
-            $this->settings = (array) $db->fetchPairs($query);
-        }
-
-        return $this->settings;
-    }
-
     public function getActivitylogNeighbors($id, $type = null, $name = null)
     {
         $db = $this->db();
@@ -268,7 +222,7 @@ class Db extends DbConnection
     public function fetchActivityLogChecksumById($id, $binary = true)
     {
         $sql = sprintf(
-            'SELECT %s AS checksum FROM director_activity_log WHERE id = %d',
+            'SELECT' . ' %s AS checksum FROM director_activity_log WHERE id = %d',
             $this->dbHexFunc('checksum'),
             (int) $id
         );
@@ -292,6 +246,8 @@ class Db extends DbConnection
 
     public function fetchActivityLogEntry($checksum)
     {
+        $db = $this->db();
+
         $sql = 'SELECT id, object_type, object_name, action_name,'
              . ' old_properties, new_properties, author, change_time,'
              . ' %s AS checksum, %s AS parent_checksum'
@@ -303,9 +259,8 @@ class Db extends DbConnection
             $this->dbHexFunc('parent_checksum')
         );
 
-        return $this->db()->fetchRow(
-            $sql,
-            $this->quoteBinary(Util::hex2binary($checksum))
+        return $db->fetchRow(
+            $db->quoteInto($sql, $this->quoteBinary(Util::hex2binary($checksum)))
         );
     }
 
@@ -402,105 +357,6 @@ class Db extends DbConnection
         return $objects;
     }
 
-    public function fetchLatestImportedRows($source, $desiredColumns = null)
-    {
-        $db = $this->db();
-        if ($desiredColumns === null) {
-            $columns = null;
-        } else {
-            $columns = array();
-            foreach ($desiredColumns as $column) {
-                if (false === ($pos = strpos($column, '.'))) {
-                    $columns[$column] = $column;
-                } else {
-                    $column = substr($column, 0, $pos);
-                    $columns[$column] = $column;
-                }
-            }
-        }
-
-        $lastRun = $db->select()->from(
-            array('r' => 'import_run'),
-            array('checksum' => $this->dbHexFunc('r.rowset_checksum'))
-        );
-
-        if (is_int($source) || ctype_digit($source)) {
-            $lastRun->where('r.source_id = ?', $source);
-        } else {
-            $lastRun->where('r.source_name = ?', $source);
-        }
-
-        $lastRun->order('r.start_time DESC')->limit(1);
-        $checksum = $db->fetchOne($lastRun);
-
-        return $this->fetchImportedRowsetRows($checksum, $columns);
-    }
-
-    public function fetchImportedRowsetRows($checksum, $columns, $filter = null)
-    {
-        $db = $this->db();
-        $binchecksum = Util::hex2binary($checksum);
-
-        $query = $db->select()->from(
-            array('rsr' => 'imported_rowset_row'),
-            array(
-                'object_name'    => 'r.object_name',
-                'property_name'  => 'p.property_name',
-                'property_value' => 'p.property_value',
-                'format'         => 'p.format'
-            )
-        )->join(
-            array('r' => 'imported_row'),
-            'rsr.row_checksum = r.checksum',
-            array()
-        )->join(
-            array('rp' => 'imported_row_property'),
-            'r.checksum = rp.row_checksum',
-            array()
-        )->join(
-            array('p' => 'imported_property'),
-            'p.checksum = rp.property_checksum',
-            array()
-        )->where('rsr.rowset_checksum = ?', $this->quoteBinary($binchecksum))->order('r.object_name');
-
-        if ($columns === null) {
-            $columns = $this->listImportedRowsetColumnNames($checksum);
-        } else {
-            $query->where('p.property_name IN (?)', $columns);
-        }
-
-        $result = array();
-        $empty = (object) array();
-        foreach ($columns as $k => $v) {
-            $empty->$k = null;
-        }
-
-        foreach ($db->fetchAll($query) as $row) {
-            if (! array_key_exists($row->object_name, $result)) {
-                $result[$row->object_name] = clone($empty);
-            }
-
-            if ($row->format === 'json') {
-                $result[$row->object_name]->{$row->property_name} = json_decode($row->property_value);
-            } else {
-                $result[$row->object_name]->{$row->property_name} = $row->property_value;
-            }
-        }
-
-        if ($filter) {
-            $filtered = array();
-            foreach ($result as $key => $row) {
-                if ($filter->matches($row)) {
-                    $filtered[$key] = $row;
-                }
-            }
-
-            return $filtered;
-        }
-
-        return $result;
-    }
-
     public function getLatestImportedChecksum($source)
     {
         $db = $this->db();
@@ -564,26 +420,6 @@ class Db extends DbConnection
         return $result;
     }
 
-    public function listImportedRowsetColumnNames($checksum)
-    {
-        $db = $this->db();
-
-        $query = $db->select()->distinct()->from(
-            array('p' => 'imported_property'),
-            'property_name'
-        )->join(
-            array('rp' => 'imported_row_property'),
-            'rp.property_checksum = p.checksum',
-            array()
-        )->join(
-            array('rsr' => 'imported_rowset_row'),
-            'rsr.row_checksum = rp.row_checksum',
-            array()
-        )->where('rsr.rowset_checksum = ?', $this->quoteBinary(Util::hex2binary($checksum)));
-
-        return $db->fetchCol($query);
-    }
-
     public function enumCommands()
     {
         return $this->enumIcingaObjects('command');
@@ -608,6 +444,15 @@ class Db extends DbConnection
         return $this->enumIcingaObjects('command', $filters);
     }
 
+    public function enumEventcommands()
+    {
+        $filters = array(
+            'methods_execute = ?' => 'PluginEvent',
+
+        );
+        return $this->enumIcingaObjects('command', $filters);
+    }
+
     public function enumNotificationCommands()
     {
         $filters = array(
@@ -785,9 +630,23 @@ class Db extends DbConnection
         return $this->db()->fetchAll($select);
     }
 
-    public function isPgsql()
+    public function fetchDistinctServiceVars()
     {
-        return $this->getDbType() === 'pgsql';
+        $select = $this->db()->select()->distinct()->from(
+            array('sv' => 'icinga_service_var'),
+            array(
+                'varname'  => 'sv.varname',
+                'format'   => 'sv.format',
+                'caption'  => 'df.caption',
+                'datatype' => 'df.datatype'
+            )
+        )->joinLeft(
+            array('df' => 'director_datafield'),
+            'df.varname = sv.varname',
+            array()
+        )->order('varname');
+
+        return $this->db()->fetchAll($select);
     }
 
     public function dbHexFunc($column)
@@ -834,6 +693,9 @@ class Db extends DbConnection
         return $db->fetchPairs($query);
     }
 
+    /**
+     * @return DirectorDeploymentLog[]
+     */
     public function getUncollectedDeployments()
     {
         $db = $this->db();
@@ -847,4 +709,16 @@ class Db extends DbConnection
 
         return DirectorDeploymentLog::loadAll($this, $query, 'stage_name');
     }
+
+    public function hasUncollectedDeployments()
+    {
+        $db = $this->db();
+        $query = $db->select()
+            ->from('director_deployment_log', array('cnt' => 'COUNT(*)'))
+            ->where('stage_name IS NOT NULL')
+            ->where('stage_collected IS NULL')
+            ->where('startup_succeeded IS NULL');
+
+        return $db->fetchOne($query) > 0;
+    }
 }
