@@ -40,11 +40,13 @@ class Migration
 
         try {
             foreach ($queries as $query) {
-                $db->exec($query);
+                if (preg_match('/^(?:OPTIMIZE|EXECUTE) /i', $query)) {
+                    $db->query($query);
+                } else {
+                    $db->exec($query);
+                }
             }
-
         } catch (Exception $e) {
-
             throw new IcingaException(
                 'Migration %d failed (%s) while running %s',
                 $this->version,
